@@ -14,6 +14,7 @@
 #include "sweepscanner.h"
 #include <ktabwidget.h>
 #include <qlayout.h>
+#include <qhbox.h>
 #include <qlabel.h>
 #include <qcheckbox.h>
 #include <kpushbutton.h>
@@ -21,7 +22,7 @@
 #include <qhbox.h>
 #include <qvbox.h>
 #include <qsizepolicy.h>
-#include <qlineedit.h>
+#include <klineedit.h>
 #include <qcursor.h>
 #include <qtooltip.h>
 #include <qfile.h>
@@ -32,6 +33,7 @@
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
 #include <kurlrequester.h>
+#include <qregexp.h>
 #include <unistd.h>
 #include <errno.h>
 #include <qdatetime.h>
@@ -57,17 +59,17 @@
 int SweeperWizard::CommonColumns = FileName | FileDir | ModTime | Owner | Group | Permissions;
 
 SweeperWizard::ScanOpts SweeperWizard::scanopts[] = {
-	{SweepScanner::EMPTY_FILE, i18n("Empty files"),                 CommonColumns, i18n("Search for zero-length files")},
-	{SweepScanner::EMPTY_DIR, i18n("Empty directories"),            CommonColumns, QString::null},
-        {SweepScanner::BROKEN_SYMLINK, i18n("Broken symlinks"),         CommonColumns | Target, i18n("Search for symbolic links pointing to non-existing files")},
-        {SweepScanner::BROKEN_EXEC, i18n("Broken executables"),         CommonColumns | Target, i18n("Search for executables in PATH with missing libraries")},
-        {SweepScanner::BACKUP_FILE, i18n("Backup files"),               CommonColumns | Size, i18n("Search for files with common backup extensions (.bak, .tmp and others)")},
-        {SweepScanner::ORPHAN_FILE, i18n("Orphaned files"),        CommonColumns | Size, i18n("Search for files that cannot be found in installed packages database of your system")},
-        {SweepScanner::DEAD_MENUENTRY, i18n("Dead menu entries"),       CommonColumns | Size | Target, i18n("Search for desktop menu entires (.desktop files) pointing to non-existing executables")},
-	{SweepScanner::DEAD_THUMBNAIL, i18n("Obsolete thumbnails"),     CommonColumns | Size | Target, i18n("Search for thumbnails pointing to non-existing images")},
-        {SweepScanner::DUPLICATED_FILE, i18n("Duplicated files"),       CommonColumns | Size, i18n("Search for files with the same contents")},
-	//{SweepScanner::OLD_FILE, i18n("Files not accessed recently"),   CommonColumns | AccessTime | Size, i18n("Search for files not accessed for specified amount of time")},
-	{SweepScanner::UNUSED_LOCALE, i18n("Unused locales"),           CommonColumns | Size, i18n("Search for .po locale files for languages not used by your system")},
+	{SweepScanner::EMPTY_FILE, I18N_NOOP("Empty files"),                 CommonColumns, I18N_NOOP("Search for zero-length files")},
+	{SweepScanner::EMPTY_DIR, I18N_NOOP("Empty directories"),            CommonColumns, I18N_NOOP("Search for empty directories")},
+        {SweepScanner::BROKEN_SYMLINK, I18N_NOOP("Broken symlinks"),         CommonColumns | Target, I18N_NOOP("Search for symbolic links pointing to non-existing files")},
+        {SweepScanner::BROKEN_EXEC, I18N_NOOP("Broken executables"),         CommonColumns | Target, I18N_NOOP("Search for executables in PATH with missing libraries")},
+        {SweepScanner::BACKUP_FILE, I18N_NOOP("Backup files"),               CommonColumns | Size, I18N_NOOP("Search for files with common backup extensions (.bak, .tmp and others)")},
+        {SweepScanner::ORPHAN_FILE, I18N_NOOP("Orphaned files"),        CommonColumns | Size, I18N_NOOP("Search for files that cannot be found in installed packages database of your system")},
+        {SweepScanner::DEAD_MENUENTRY, I18N_NOOP("Dead menu entries"),       CommonColumns | Size | Target, I18N_NOOP("Search for desktop menu entires (.desktop files) pointing to non-existing executables")},
+	{SweepScanner::DEAD_THUMBNAIL, I18N_NOOP("Obsolete thumbnails"),     CommonColumns | Size | Target, I18N_NOOP("Search for thumbnails pointing to non-existing images")},
+        {SweepScanner::DUPLICATED_FILE, I18N_NOOP("Duplicated files"),       CommonColumns | Size, I18N_NOOP("Search for files with the same contents")},
+	//{SweepScanner::OLD_FILE, I18N_NOOP("Files not accessed recently"),   CommonColumns | AccessTime | Size, I18N_NOOP("Search for files not accessed for specified amount of time")},
+	{SweepScanner::UNUSED_LOCALE, I18N_NOOP("Unused locales"),           CommonColumns | Size, I18N_NOOP("Search for .po locale files for languages not used by your system")},
 	{SweepScanner::UNUSED_LOCALE, QString::null, 0, QString::null}
 };
 
@@ -192,9 +194,9 @@ void SweeperWizard::setupStartPage()
 	// create checkboxes for search categories
 	for (int i=0; scanopts[i].optname!=QString::null; i++)
 	{
-		QCheckBox *cb = cb_scan[scanopts[i].ftype] = new QCheckBox(scanopts[i].optname, b_group0);
+		QCheckBox *cb = cb_scan[scanopts[i].ftype] = new QCheckBox(i18n(scanopts[i].optname), b_group0);
 		if (scanopts[i].info != QString::null)
-			QToolTip::add(cb, scanopts[i].info);
+			QToolTip::add(cb, i18n(scanopts[i].info));
 		connect(cb, SIGNAL(toggled(bool)), this, SLOT(scanCategoryToggled(bool)));
 	}
 	//KPushButton *b_allscans = new KPushButton(i18n("All"), b_group0);
@@ -243,12 +245,38 @@ void SweeperWizard::setupResultsPage()
         resultstabs = new KTabWidget(resultspage);
 	box0->addWidget(resultstabs);
 
+
+	//
+	// more "select" option (regular expression support)
+	moreopt = new QHBoxLayout(NULL, 0, 5, NULL);
+	box0->addLayout(moreopt);
+	b_regexclr = new KPushButton(KGlobal::iconLoader()->loadIconSet("locationbar_erase", KIcon::Small), QString::null, resultspage);
+	regex_edit = new KLineEdit(resultspage);
+	regex_edit->setMaxLength(512);
+	QToolTip::add(regex_edit, i18n("Enter text or regular expression"));
+	connect(regex_edit, SIGNAL(textChanged(const QString&)), this, SLOT(regexChanged(const QString&)));
+	cb_regexpcase = new QCheckBox(i18n("Case sensitive"), resultspage);
+	cb_regexpgreedy = new QCheckBox(i18n("Greedy match"), resultspage);
+	cb_regexpcase->setChecked(true);  //case sensitive by default
+	cb_regexpgreedy->setChecked(true);  //greedy matching by default
+	b_regexpselect = new KPushButton(i18n("Select"), resultspage);
+	b_regexpunselect = new KPushButton(i18n("Unselect"), resultspage);
+	moreopt->addWidget(b_regexclr);
+	moreopt->addWidget(regex_edit);
+	moreopt->addWidget(cb_regexpcase);
+	moreopt->addWidget(cb_regexpgreedy);
+	moreopt->addWidget(b_regexpselect);
+	moreopt->addWidget(b_regexpunselect);
+	
 	QHBoxLayout *box1 = new QHBoxLayout(NULL, 0, 5, NULL);
+	b_more = new KPushButton(">>", resultspage);
+	b_more->setToggleButton(true);
 	b_selectall = new KPushButton(i18n("Select all"), resultspage);
 	b_clearall = new KPushButton(i18n("Clear all"), resultspage);
 	b_saveresrep = new KPushButton(KGlobal::iconLoader()->loadIconSet("filesaveas", KIcon::Small), i18n("Save report"), resultspage);
 	summary = new SummaryLabel(resultspage);
 	b_stop = new KPushButton(KGlobal::iconLoader()->loadIconSet("stop", KIcon::Small), i18n("Stop"), resultspage);
+	box1->addWidget(b_more);
 	box1->addWidget(b_selectall);
 	box1->addWidget(b_clearall);
 	box1->addWidget(b_saveresrep);
@@ -257,7 +285,11 @@ void SweeperWizard::setupResultsPage()
 	box1->addStretch(1);
 	box1->addWidget(summary);
 
+	connect(b_more, SIGNAL(clicked()), this, SLOT(moreClicked()));
 	connect(b_selectall, SIGNAL(clicked()), this, SLOT(selectAllClicked()));
+	connect(b_regexclr, SIGNAL(clicked()), regex_edit, SLOT(clear()));
+	connect(b_regexpselect, SIGNAL(clicked()), this, SLOT(selectRegExpClicked()));
+	connect(b_regexpunselect, SIGNAL(clicked()), this, SLOT(unselectRegExpClicked()));
 	connect(b_clearall, SIGNAL(clicked()), this, SLOT(clearAllClicked()));
 	connect(b_saveresrep, SIGNAL(clicked()), this, SLOT(saveReport()));
 	connect(b_stop, SIGNAL(clicked()), this, SLOT(stopClicked()));
@@ -266,6 +298,10 @@ void SweeperWizard::setupResultsPage()
 
 	for (int i=0; i<SweepScanner::NONE; i++)
 		views[i] = NULL;
+
+	moreClicked(); //ensures additional options are initially hidden
+	regexChanged(""); //ensures select/unselect buttons are initially disabled for empty regex
+	
         addPage(resultspage, i18n("Scan results"));
 }
 
@@ -390,7 +426,7 @@ void SweeperWizard::showPage(QWidget *w)
 				views[n] = new FileListView(resultstabs, prev);
 				views[n]->setupColumns(columns);
 				connect(views[n], SIGNAL(openDirectory(const QString &)), this, SLOT(openDirectory(const QString &)));
-				resultstabs->insertTab(views[n], scanopts[i].optname);
+				resultstabs->insertTab(views[n], i18n(scanopts[i].optname));
 				prev = views[n];
 				connect(prev, SIGNAL(selectionChanged(int, unsigned int)), summary, SLOT(updateSelectedInfo(int, unsigned int)));
 			}
@@ -411,12 +447,7 @@ void SweeperWizard::showPage(QWidget *w)
 		if (scanner->start(path))
 		{
 
-			setNextEnabled(resultspage, false);
-			setBackEnabled(resultspage, false);
-			b_selectall->setDisabled(true);
-			b_clearall->setDisabled(true);
-			b_saveresrep->setDisabled(true);
-			b_stop->setDisabled(false);
+			enableResultsPageElements(false);
 			setCursor(QCursor(Qt::WaitCursor));
 		}
 		else
@@ -543,6 +574,30 @@ void SweeperWizard::showPage(QWidget *w)
         KWizard::showPage(w);
 }
 
+void SweeperWizard::enableResultsPageElements(bool f)
+{
+	setNextEnabled(resultspage, f);
+	setBackEnabled(resultspage, f);
+
+	b_stop->setDisabled(f);
+
+	f = !f;
+
+	b_selectall->setDisabled(f);
+	b_clearall->setDisabled(f);
+	b_saveresrep->setDisabled(f);
+
+	bool regex_empty = regex_edit->text().isEmpty();
+	
+	b_more->setDisabled(f);
+	b_regexclr->setDisabled(f);
+	regex_edit->setDisabled(f);
+	b_regexpselect->setDisabled(f || regex_empty);
+	b_regexpunselect->setDisabled(f || regex_empty);
+	cb_regexpcase->setDisabled(f);
+	cb_regexpgreedy->setDisabled(f);
+}
+
 void SweeperWizard::addFileEntry(SweepScanner::FileType t, SweepFileInfo *f)
 {
 	if (views[t])
@@ -572,12 +627,7 @@ void SweeperWizard::scannerFinished(bool status)
 		progress->close();
 		progress = NULL;
 	}
-	setNextEnabled(resultspage, true);
-	setBackEnabled(resultspage, true);
-	b_selectall->setDisabled(false);
-	b_clearall->setDisabled(false);
-	b_saveresrep->setDisabled(false);
-	b_stop->setDisabled(true);
+	enableResultsPageElements(true);
 	setCursor(QCursor());
 }
 
@@ -597,6 +647,13 @@ void SweeperWizard::openDirectory(const QString &path)
         proc->start(KProcess::DontCare, KProcess::NoCommunication);
 }
 
+void SweeperWizard::regexChanged(const QString &str)
+{
+	const bool f = str.isEmpty();
+	b_regexpselect->setDisabled(f);
+	b_regexpunselect->setDisabled(f);
+}
+
 void SweeperWizard::selectAllClicked()
 {
 	if (FileListView *w = dynamic_cast<FileListView*>(resultstabs->currentPage()))
@@ -610,6 +667,67 @@ void SweeperWizard::selectAllClicked()
 	}
 }
 
+void SweeperWizard::moreClicked()
+{
+	const bool f = b_more->isOn();
+	b_more->setText(f ? "<<" : ">>");
+	b_regexclr->setShown(f);
+	regex_edit->setShown(f);
+	cb_regexpcase->setShown(f);
+	cb_regexpgreedy->setShown(f);
+	b_regexpselect->setShown(f);
+	b_regexpunselect->setShown(f);
+}
+
+void SweeperWizard::selectRegExpClicked()
+{
+	if (FileListView *w = dynamic_cast<FileListView*>(resultstabs->currentPage()))
+	{
+		if (!regex_edit->text().isEmpty())
+		{
+			QRegExp rx(regex_edit->text(), cb_regexpcase->isChecked());
+			rx.setMinimal(!cb_regexpgreedy->isChecked());
+			if (rx.isValid())
+			{
+				setCursor(QCursor(Qt::WaitCursor));
+				w->select(rx);
+				setCursor(QCursor());
+			}
+			else
+			{
+				invalidRegExp();
+			}
+		}
+	}
+}
+
+void SweeperWizard::unselectRegExpClicked()
+{
+	if (FileListView *w = dynamic_cast<FileListView*>(resultstabs->currentPage()))
+	{
+		if (!regex_edit->text().isEmpty())
+		{
+			QRegExp rx(regex_edit->text(), cb_regexpcase->isChecked());
+			rx.setMinimal(!cb_regexpgreedy->isChecked());
+			if (rx.isValid())
+			{
+				setCursor(QCursor(Qt::WaitCursor));
+				w->unselect(rx);
+				setCursor(QCursor());
+			}
+			else
+			{
+				invalidRegExp();
+			}
+		}
+	}
+}
+
+void SweeperWizard::invalidRegExp()
+{
+	KMessageBox::sorry(this, i18n("Invalid regular expression"));
+}
+
 void SweeperWizard::clearAllClicked()
 {
 	if (FileListView *w = dynamic_cast<FileListView*>(resultstabs->currentPage()))
